/*
 * Developed by Hillstone Software. http://www.hillstone-software.com
 * HS NTP Protocol Module interface definitions
 */
#ifndef _HS_NTP_IF_H
#define _HS_NTP_IF_H
#include <windows.h>

/* timer callback function */
typedef void timer_callback_t(void *arg);

/* start timer function, 
 * secs - timeout value in seconds, 
 * callback -function (in HS NTP module code) to call on timer expiry 
 * return - timer handle, tol be used by HS NTP to stop timer */
typedef long start_timer_t(unsigned long secs, timer_callback_t *callback);

/* stop timer function, timer_handle - timer handle obtained from call to start_timer */
typedef void stop_timer_t(long timer_handle);

/* event callback function, called by HS NTP module to report events to user code 
 * handle - user session handle 
 * ev - event id 
 * arg1, arg2 - depends on event, see HS NTP manual */
typedef int event_callback_t(long user_handle, int ev, long arg1, long arg2);

/* api block */
typedef struct
{
    start_timer_t       *start_timer;   // start timer function
    stop_timer_t        *stop_timer;    // stop timer function
    event_callback_t    *event_cb;      // event callback function
} hs_ntp_api_t;

#define HS_NTP_MAX_SRVNAME         80  // maximum size of server name string including null terminator
#define HS_NTP_MAX_IP              16  // maximum size of ip address string including null terminator

/* session parameters */
typedef struct
{
	/* either name or IP address may be specified, if not specified supply NULL pointer */
	/* if srv_name is NULL, try to connect using srv_ip - which is a string in the form x.x.x.x */
	/* if srv_name is not null, HS NTP will try to resolve name to IP address */	
	unsigned char	srv_name[HS_NTP_MAX_SRVNAME];		// NTP server name, null terminated string
	unsigned char	srv_ip[HS_NTP_MAX_IP];		        // NTP server ip address, null terminated string

	unsigned short	source_port;						// NTP client source UDP port

    void			*user_data;		// user data association with this session (user sesison reference) not modified by NTP module
} hs_ntp_session_t;

/* HS NTP return codes */
typedef enum
{
    HS_NTP_RC_OK,                      // success
    HS_NTP_RC_NOTINIT,                 // HS NTP module not initialised
    HS_NTP_RC_ALRINIT,                 // HS NTP already initialised
    HS_NTP_RC_SOCKINIT_FAIL,           // socket layer initialisation failure
    HS_NTP_RC_UDP_SOCK_OPEN,		   // failed to open UDP socket
    HS_NTP_RC_UDP_SOCK_SEND,		   // could not send UDP packet
    HS_NTP_RC_INV_PAR,                 // invalid parameters
    HS_NTP_RC_BUSY, 		           // exceeded maximum number of open sessions
    HS_NTP_RC_NONAME,                  // socket layer could not resolve server name
    HS_NTP_RC_TIMERFAIL,               // timer allocation failed
    
    HS_NTP_SRVERR_LEN,				   // invalid NTP reply packet length
    HS_NTP_SRVERR_UNSYNC,			   // server time not synchronised
    HS_NTP_SRVERR_MODE,				   // invalid mode
    
    HS_NTP_RC_NUM_EVENTS,              // internal significance to HS NTP module only
} hs_ntp_rc_t;

/* callback events */
typedef enum
{
	HS_NTP_USR_EV_SRVRESP,				// NTP server response
	HS_NTP_USR_EV_SRVRESPERR,			// NTP server response error
	HS_NTP_USR_EV_TIMEDOUT,				// Timed out waiting for server response
	HS_NTP_USR_EV_DNS_ERROR,			// could not resolve server ip address
	HS_NTP_USR_EV_SOCK_ERROR,			// socket layer error
	HS_NTP_USR_EV_TMR_ERROR,			// timer error
} hs_ntp_usr_ev_t;

/* NTP response information */
typedef struct
{
	unsigned __int64	T1;				// time request sent by client
	unsigned __int64	T2; 			// time request received at server
	unsigned __int64	T3;				// time reply sent by server	
	unsigned __int64	T4;				// time reply received at client
		
	long 				offset_seconds;	// seconds part of offset
	long 				offset_milliseconds;// fractions part of offset
	
	SYSTEMTIME 			st1;			// time request sent by client
	SYSTEMTIME 			st2;			// time request received at server
	SYSTEMTIME 			st3;			// time reply sent by server
	SYSTEMTIME 			st4;			// time reply received at client
} hs_ntp_info_t;

/* 
 * Initialise HS NTP protocol module
 */
extern int HsNtpInit(hs_ntp_api_t *api);

/*
 * De-initialise HS NTP module and release all resources
 */
extern int HsNtpDestroy(void);

/*
 * Initiate Time request using NTP protocol
 */
extern int HsNtpGetTime(hs_ntp_session_t *s);

/* 
 * Convert integer return code into description string
 */
extern unsigned char *HsNtpGetErrStr(int rc);

#endif /* _HS_NTP_IF_H */
