/*
 * Developed by Hillstone Software. http://www.hillstone-software.com
 * TFTP library main API definitions
 */
#ifndef HS_TFTP_API_H
#define HS_TFTP_API_H

#ifndef true
#define true	1
#define false	0
#endif

typedef enum
{
	TFTP_OP_SEND_FILE,
	TFTP_OP_GET_FILE,
} tctp_operation_t;

typedef enum
{
	HS_TFTP_ERR_NOT_DEFINED = 0,
	HS_TFTP_ERR_FILE_NOT_FOUND,
	HS_TFTP_ERR_ACCESS_VIOLATION,
	HS_TFTP_ERR_DISK_FULL,
	HS_TFTP_ERR_ILLEGAL_OPERATION,
	HS_TFTP_ERR_UNKNOWN_TID,
	HS_TFTP_ERR_ALREADY_EXISTS,
	HS_TFTP_ERR_NO_SUCH_USER,
	HS_TFTP_ERR_OACK,
} hs_tftp_prot_err_t;

/* return codes from HS TFTP to user calls */
typedef enum
{
	HS_TFTP_RC_FAIL		= 0,			// general failure, no additional information available
	HS_TFTP_RC_OK,						// success
	HS_TFTP_RC_INVALID_PAR,				// invalid call parameters
	HS_TFTP_RC_NO_MEM,					// no free memory
	HS_TFTP_SOCK_STARTUP_FAIL,			// Socket layer startup failed
	HS_TFTP_RC_INIT,					// already initialised
	HS_TFTP_RC_NOT_INIT,				// not initialised
	HS_TFTP_RC_NO_FREE_CTX,				// no free tftp contexts
	HS_TFTP_RC_UDP_SOCK_OPEN,			// UDP socket open failed
	HS_TFTP_RC_UDP_SOCK_SEND,			// UDP socket send failed

    HS_TFTP_RC_FOPEN,                   // file open error

	HS_TFTP_MAX_RC,
};

/* event codes passed in hs_tftp_ev_fn_t callback */
typedef enum
{
	HS_TFTP_EV_ERR_TMOUT,		// connection timed out
	HS_TFTP_RX_COMPLETE,		// file reception complete
	HS_TFTP_EV_WRITE_REQ,		// server context received write request from remote client
	HS_TFTP_EV_READ_REQ,		// server context received read request from remote client
	HS_TFTP_EV_ERR_RXED,		// ERROR packet received from remote
	HS_TFTP_EV_ERR_OACK,		// TFTP session closed, invalid OACK packet received from server
} ev_code_t;

/* server mode errors - from application to HS TFTP library */
typedef enum
{
	HS_TFTP_SERVER_ERR_FIO,				// FILE I/o error
	HS_TFTP_SERVER_ERR_FNOTFOUND,		// file not found
	HS_TFTP_SERVER_ERR_USER,			// user defined error, send supplied error string
} srv_err_t;

/* function in user code called by HS TFTP to report connection related events 
 * i.e. errors and normal completion of file transfers 
 * handle - conneciton handle 
 * ev_code - event code (see above) */
typedef long hs_tftp_ev_fn_t(long handle, int ev_code, long arg1, long arg2);

/* function in user code called by HS TFTP to report server connection related events 
 * handle - conneciton handle 
 * ev_code - event code (see above) */
typedef long hs_tftp_srv_ev_fn_t(long handle, int ev_code, long arg1, long arg2);

/* function prototype used to get memory */
/* handle - connection handle (obtained with HsTftpGetFile) 
/* length - length of file - memory size required and how much actually returned
/* cmd - optional information from user code
/* return: pointer to memory or null if insufficient space */
typedef unsigned char *hs_tftp_get_buf_t(long handle, unsigned int *length, int *cmd);

/* function prototype used by HS TFTP to start a timer 
 * secs - number of seconds to set timer to
 * handle - connection handle
 * return: timer handle or NULL if user code could not start timer */
typedef long hs_tftp_start_timer_t(long handle, unsigned long secs);

/* function in used code used by HS TFTP to stop previously started timer */
/* Note that TFTP HS assumes that timer is stopped anyway if it has fired (expired) */
/* handle - timer handle */
typedef void hs_tftp_stop_timer_t(long handle);

/* initialisation structure - used to initialise HS TFTP library with HsTftpInit call, before it can be used */
typedef struct 
{	
	hs_tftp_get_buf_t			*get_tx_buffer;		// Function in user code used to get memory buffer (for transmission)
	hs_tftp_get_buf_t			*get_rx_buffer;		// Function in user code used to get memory buffer (for reception)
	hs_tftp_start_timer_t		*start_timer_fn; 	// Function in user code used by HS TFTP to start a timer
	hs_tftp_stop_timer_t		*stop_timer_fn; 	// Function in user code used by HS TFTP to stop a timer

	unsigned short int			max_blksize;		// maximum possible TFTP data block size, 0=use default, otherwise 8 to 65464 is the valid range
} tftp_init_t;

/*
 * Initialisation function must be called first before any other API functions can be used
 */
extern int HsTftpInit(tftp_init_t *init);

/*
 * Deallocate resources and close HS TFTP services
 */
extern void HsTftpDestroy(void);

/*
 * Initiate Read file or Write file transfer operation
 */
extern
int HsTftpTransfer(int				operation,		// packet type
				 unsigned long 		dest_ip, 		// remote end IP address
				 unsigned char 		*filename, 		// filename (0 terminated) to be used to store file at remote end
				 hs_tftp_ev_fn_t	*callback_fn,	// event callback (used for indication of completion or error)
				 unsigned short		tftp_port,		// initial TFTP port of server
				 
				 unsigned short int	blocksize,		// 0=block size option not supported (default 512 bytes used) 
				 									// or blocksize to use in option negitiation, valid values 8 to 65464
				 									
				 long				*handle,		// Connection handle returned after transfer initiated
				 long				user_handle);	// upper layer context

/*
 * Abort current operation and cleanly disconnect remote end based on passed connection handle
 */
extern int HsTftpAbort(long handle);

/* Function called from user code when timer previously started by HS TFTP has expired
/* timer_handle - handle of timer expired */
extern void HsTftpTimerExpired(long timer_handle);

/*
 * Start TFTP server
 */ 
extern int HsTftpStartServer(hs_tftp_srv_ev_fn_t *callback_fn,	// event callback (used for server mode)
							unsigned short int	blocksize,		// 0=block size option not supported (default 512 bytes used) 
							 									// or blocksize to use in option negitiation, valid values 8 to 65464
							unsigned short		tftp_port);		// initial TFTP port of server
/*
 * Initiate Write file transfer operation (Server Mode)
 */
extern
int HsTftpServerStartReceive(	hs_tftp_ev_fn_t		*callback_fn,	// event callback (used for infication of completion or error)
								long				*handle,		// Connection handle returned after transfer initiated
								long				user_handle);	// upper layer context handle

/*
 * Initiate Read file transfer operation (Server Mode)
 */
extern
int HsTftpServerStartSend(hs_tftp_ev_fn_t	*callback_fn,	// event callback (used for infication of completion or error)
							long			*handle,		// Connection handle returned after transfer initiated
							long			user_handle);	// upper layer context handle
/*
 * Get error string
 */
extern unsigned char *HsTftpErrStr(int rc);

/* Reject request (server mode) */
extern void HsTftpRejectRq(int rq, int reason, unsigned char *str);

#endif /* HS_TFTP_API_H */

